from enum import IntEnum


class Squads(IntEnum):
    NO_SQUAD = 0,
    ALPHA = 1,
    BRAVO = 2,
    CHARLIE = 3,
    DELTA = 4,
    ECHO = 5,
    FOXTROT = 6,
    GOLF = 7,
    HOTEL = 8,
    INDIA = 9,
    JULIETT = 10,
    KILO = 11,
    LIMA = 12,
    MIKE = 13,
    NOVEMBER = 14,
    OSCAR = 15,
    PAPA = 16,
    QUEBEC = 17,
    ROMEO = 18,
    SIERRA = 19,
    TANGO = 20,
    UNIFORM = 21,
    WHISKEY = 22,
    XRAY = 23,
    YANKEE = 24,
    ZULU = 25,
    ASH = 26,
    BAKER = 27,
    CAST = 28,
    DIVER = 29,
    EAGLE = 30,
    FISHER = 31,
    GEORGE = 32,
    HANOVER = 33,
    ICE = 34,
    JAKE = 35,
    KING = 36,
    LASH = 37,
    MULE = 38,
    NEPTUNE = 39,
    OSTEND = 40,
    PAGE = 41,
    QUAIL = 42,
    RAFT = 43,
    SCOUT = 44,
    TARE = 45,
    UNIT = 46,
    WILLIAM = 47,
    XAINTRIE = 48,
    YOKE = 49,
    ZEBRA = 50,
    ACE = 51,
    BEER = 52,
    CAST2 = 53,
    DUFF = 54,
    EDWARD = 55,
    FREDDY = 56,
    GUSTAV = 57,
    HENRY = 58,
    IVAR = 59,
    JAZZ = 60,
    KEY = 61,
    LINCOLN = 62,
    MARY = 63,
    NORA = 64
